package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.KeyParameter;

public class SkipjackEngine implements BlockCipher {
   static final int BLOCK_SIZE = 8;
   static short[] ftable = new short[]{163, 215, 9, 131, 248, 72, 246, 244, 179, 33, 21, 120, 153, 177, 175, 249, 231, 45, 77, 138, 206, 76, 202, 46, 82, 149, 217, 30, 78, 56, 68, 40, 10, 223, 2, 160, 23, 241, 96, 104, 18, 183, 122, 195, 233, 250, 61, 83, 150, 132, 107, 186, 242, 99, 154, 25, 124, 174, 229, 245, 247, 22, 106, 162, 57, 182, 123, 15, 193, 147, 129, 27, 238, 180, 26, 234, 208, 145, 47, 184, 85, 185, 218, 133, 63, 65, 191, 224, 90, 88, 128, 95, 102, 11, 216, 144, 53, 213, 192, 167, 51, 6, 101, 105, 69, 0, 148, 86, 109, 152, 155, 118, 151, 252, 178, 194, 176, 254, 219, 32, 225, 235, 214, 228, 221, 71, 74, 29, 66, 237, 158, 110, 73, 60, 205, 67, 39, 210, 7, 212, 222, 199, 103, 24, 137, 203, 48, 31, 141, 198, 143, 170, 200, 116, 220, 201, 93, 92, 49, 164, 112, 136, 97, 44, 159, 13, 43, 135, 80, 130, 84, 100, 38, 125, 3, 64, 52, 75, 28, 115, 209, 196, 253, 59, 204, 251, 127, 171, 230, 62, 91, 165, 173, 4, 35, 156, 20, 81, 34, 240, 41, 121, 113, 126, 255, 140, 14, 226, 12, 239, 188, 114, 117, 111, 55, 161, 236, 211, 142, 98, 139, 134, 16, 232, 8, 119, 17, 190, 146, 79, 36, 197, 50, 54, 157, 207, 243, 166, 187, 172, 94, 108, 169, 19, 87, 37, 181, 227, 189, 168, 58, 1, 5, 89, 42, 70};
   private int[] key0;
   private int[] key1;
   private int[] key2;
   private int[] key3;
   private boolean encrypting;

   private int g(int k, int w) {
      int var10000 = w;
      w = w >> 8 & 255;
      int var3 = var10000 & 255;
      short[] var4;
      short[] var6 = var4 = ftable;
      var10000 = var6[(var3 ^= var6[(w ^= var6[var3 ^ this.key0[k]]) ^ this.key1[k]]) ^ this.key2[k]] ^ w;
      int var5 = var4[var10000 ^ this.key3[k]] ^ var3;
      return (var10000 << 8) + var5;
   }

   private int h(int k, int w) {
      int var10000 = w;
      w &= 255;
      int var3 = var10000 >> 8 & 255;
      short[] var4 = ftable;
      return ((var4[(w ^= var4[(var3 ^= var4[(w ^= var4[var3 ^ this.key3[k]]) ^ this.key2[k]]) ^ this.key1[k]]) ^ this.key0[k]] ^ var3) << 8) + w;
   }

   public void init(boolean encrypting, CipherParameters params) {
      if (!(params instanceof KeyParameter)) {
         throw new IllegalArgumentException("invalid parameter passed to SKIPJACK init - " + params.getClass().getName());
      } else {
         byte[] encrypting1 = ((KeyParameter)params).getKey();
         this.encrypting = encrypting;
         this.key0 = new int[32];
         this.key1 = new int[32];
         this.key2 = new int[32];
         this.key3 = new int[32];

         for(int params1 = 0; params1 < 32; ++params1) {
            int var3;
            this.key0[params1] = encrypting1[(var3 = params1 * 4) % 10] & 255;
            this.key1[params1] = encrypting1[(var3 + 1) % 10] & 255;
            this.key2[params1] = encrypting1[(var3 + 2) % 10] & 255;
            this.key3[params1] = encrypting1[(var3 + 3) % 10] & 255;
         }

      }
   }

   public String getAlgorithmName() {
      return "SKIPJACK";
   }

   public int getBlockSize() {
      return 8;
   }

   public int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      if (this.key1 != null) {
         if (inOff + 8 <= in.length) {
            if (outOff + 8 <= out.length) {
               if (this.encrypting) {
                  this.encryptBlock(in, inOff, out, outOff);
               } else {
                  this.decryptBlock(in, inOff, out, outOff);
               }

               return 8;
            } else {
               throw new DataLengthException("output buffer too short");
            }
         } else {
            throw new DataLengthException("input buffer too short");
         }
      } else {
         throw new IllegalStateException("SKIPJACK engine not initialised");
      }
   }

   public void reset() {
   }

   public int encryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5 = (in[inOff + 0] << 8) + (in[inOff + 1] & 255);
      int var6 = (in[inOff + 2] << 8) + (in[inOff + 3] & 255);
      int var7 = (in[inOff + 4] << 8) + (in[inOff + 5] & 255);
      int in1 = (in[inOff + 6] << 8) + (in[inOff + 7] & 255);
      inOff = 0;

      int var10000;
      for(int var8 = 0; var8 < 2; var5 = var10000) {
         int var9;
         int var10001;
         for(var9 = 0; var9 < 8; in1 = var10000) {
            in1 = (var5 = this.g(inOff, var5)) ^ in1 ^ ++inOff;
            ++var9;
            var10000 = var7;
            var10001 = var6;
            int var10002 = var5;
            var5 = in1;
            var6 = var10002;
            var7 = var10001;
         }

         var9 = 0;
         var10000 = in1;
         in1 = var5;

         for(var5 = var10000; var9 < 8; var5 = var10000) {
            var10001 = inOff;
            var6 = in1 ^ var6 ^ ++inOff;
            in1 = this.g(var10001, in1);
            ++var9;
            var10000 = var7;
            var10001 = var6;
            var6 = in1;
            in1 = var5;
            var7 = var10001;
         }

         ++var8;
         var10000 = in1;
         in1 = var5;
      }

      int var10 = outOff + 0;
      out[var10] = (byte)(var5 >> 8);
      var10 = outOff + 1;
      out[var10] = (byte)var5;
      var10 = outOff + 2;
      out[var10] = (byte)(var6 >> 8);
      var10 = outOff + 3;
      out[var10] = (byte)var6;
      var10 = outOff + 4;
      out[var10] = (byte)(var7 >> 8);
      var10 = outOff + 5;
      out[var10] = (byte)var7;
      var10 = outOff + 6;
      out[var10] = (byte)(in1 >> 8);
      var10 = outOff + 7;
      out[var10] = (byte)in1;
      return 8;
   }

   public int decryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5 = (in[inOff + 0] << 8) + (in[inOff + 1] & 255);
      int var6 = (in[inOff + 2] << 8) + (in[inOff + 3] & 255);
      int var7 = (in[inOff + 4] << 8) + (in[inOff + 5] & 255);
      int in1 = (in[inOff + 6] << 8) + (in[inOff + 7] & 255);
      inOff = 31;

      for(int var8 = 0; var8 < 2; ++var8) {
         int var9;
         int var10000;
         int var10001;
         for(var9 = 0; var9 < 8; var7 = var10000) {
            var7 = (var6 = this.h(inOff, var6)) ^ var7 ^ inOff + 1;
            --inOff;
            ++var9;
            var10000 = in1;
            var10001 = var5;
            int var10002 = var6;
            var6 = var7;
            var5 = var10002;
            in1 = var10001;
         }

         for(var9 = 0; var9 < 8; var7 = var10000) {
            var5 = var6 ^ var5 ^ inOff + 1;
            var6 = this.h(inOff, var6);
            --inOff;
            ++var9;
            var10000 = in1;
            var10001 = var5;
            var5 = var6;
            var6 = var7;
            in1 = var10001;
         }
      }

      int var10 = outOff + 0;
      out[var10] = (byte)(var5 >> 8);
      var10 = outOff + 1;
      out[var10] = (byte)var5;
      var10 = outOff + 2;
      out[var10] = (byte)(var6 >> 8);
      var10 = outOff + 3;
      out[var10] = (byte)var6;
      var10 = outOff + 4;
      out[var10] = (byte)(var7 >> 8);
      var10 = outOff + 5;
      out[var10] = (byte)var7;
      var10 = outOff + 6;
      out[var10] = (byte)(in1 >> 8);
      var10 = outOff + 7;
      out[var10] = (byte)in1;
      return 8;
   }
}
